#include "StateNameModule.hpp"

StateNameModuleImpl::StateNameModuleImpl(std::string name, const std::string & definition)
{
  setName(name);
  std::regex regex("(?:(?:\\s|\\t)*)Out\\{(.*)\\}(?:(?:\\s|\\t)*)");
  if (!util::doIfNameMatch(regex, definition, [this,&definition](auto sm)
        {
          try
          {
            outSize = std::stoi(sm.str(1));
          } catch (std::exception & e) {util::myThrow(fmt::format("{} in '{}'",e.what(),definition));}
        }))
    util::myThrow(fmt::format("invalid definition '{}'", definition));
}

torch::Tensor StateNameModuleImpl::forward(torch::Tensor input)
{
  return embeddings(input.narrow(1,firstInputIndex,1).squeeze(1));
}

std::size_t StateNameModuleImpl::getOutputSize()
{
  return outSize;
}

std::size_t StateNameModuleImpl::getInputSize()
{
  return 1;
}

void StateNameModuleImpl::addToContext(std::vector<std::vector<long>> & context, const Config & config)
{
  auto & dict = getDict();
  for (auto & contextElement : context)
    contextElement.emplace_back(dict.getIndexOrInsert(config.getState()));
}

void StateNameModuleImpl::registerEmbeddings()
{
  embeddings = register_module("embeddings", torch::nn::Embedding(getDict().size(), outSize));
}

